from __future__ import annotations
import fastgpx
import fastgpx.fastgpx
import typing
__all__ = ['Precision', 'decode', 'encode']
class Precision:
    """
    Members:
    
      Five
    
      Six
    """
    Five: typing.ClassVar[Precision]  # value = <Precision.Five: 5>
    Six: typing.ClassVar[Precision]  # value = <Precision.Six: 6>
    __members__: typing.ClassVar[dict[str, Precision]]  # value = {'Five': <Precision.Five: 5>, 'Six': <Precision.Six: 6>}
    def __eq__(self, other: typing.Any) -> bool:
        ...
    def __getstate__(self) -> int:
        ...
    def __hash__(self) -> int:
        ...
    def __index__(self) -> int:
        ...
    def __init__(self, value: int) -> None:
        ...
    def __int__(self) -> int:
        ...
    def __ne__(self, other: typing.Any) -> bool:
        ...
    def __repr__(self) -> str:
        ...
    def __setstate__(self, state: int) -> None:
        ...
    def __str__(self) -> str:
        ...
    @property
    def name(self) -> str:
        ...
    @property
    def value(self) -> int:
        ...
@typing.overload
def decode(encoded: str, precision: Precision = fastgpx.polyline.Precision.Five) -> list[fastgpx.fastgpx.LatLong]:
    ...
@typing.overload
def decode(locations: str, precision: int = 5) -> list[fastgpx.fastgpx.LatLong]:
    ...
@typing.overload
def encode(locations: list[fastgpx.fastgpx.LatLong], precision: Precision = fastgpx.polyline.Precision.Five) -> str:
    ...
@typing.overload
def encode(locations: list[fastgpx.fastgpx.LatLong], precision: int = 5) -> str:
    ...



from datetime import datetime

import fastgpx


class TestTimeBounds:

    def test_init_defaults(self):
        bounds = fastgpx.TimeBounds()
        assert bounds.is_empty()
        assert not bounds.is_range()
        assert bounds.start_time is None
        assert bounds.end_time is None

    def test_init_with_start_time(self):
        start_time = datetime.fromisoformat("2025-06-20 08:07:28+00:00")
        bounds = fastgpx.TimeBounds(start_time=start_time, end_time=None)
        assert not bounds.is_empty()
        assert not bounds.is_range()
        assert bounds.start_time == start_time
        assert bounds.end_time is None

    def test_init_with_end_time(self):
        end_time = datetime.fromisoformat("2025-06-27 13:37:00+00:00")
        bounds = fastgpx.TimeBounds(start_time=None, end_time=end_time)
        assert not bounds.is_empty()
        assert not bounds.is_range()
        assert bounds.start_time is None
        assert bounds.end_time == end_time

    def test_init_with_start_and_end_time(self):
        start_time = datetime.fromisoformat("2025-06-20 08:07:28+00:00")
        end_time = datetime.fromisoformat("2025-06-27 13:37:00+00:00")
        bounds = fastgpx.TimeBounds(start_time=start_time, end_time=end_time)
        assert not bounds.is_empty()
        assert bounds.is_range()
        assert bounds.start_time == start_time
        assert bounds.end_time == end_time

    def test_add_single_time_point(self):
        bounds = fastgpx.TimeBounds()
        time_point = datetime.fromisoformat("2025-06-20 08:07:28+00:00")
        bounds.add(time_point)
        assert not bounds.is_empty()
        assert bounds.is_range()
        assert bounds.start_time == time_point
        assert bounds.end_time == time_point

    def test_add_second_time_point(self):
        bounds = fastgpx.TimeBounds()
        time_point1 = datetime.fromisoformat("2025-06-20 08:07:28+00:00")
        time_point2 = datetime.fromisoformat("2025-06-27 13:37:00+00:00")
        bounds.add(time_point1)
        bounds.add(time_point2)
        assert not bounds.is_empty()
        assert bounds.is_range()
        assert bounds.start_time == time_point1
        assert bounds.end_time == time_point2

    def test_add_time_bounds(self):
        bounds1 = fastgpx.TimeBounds(
            start_time=datetime.fromisoformat("2025-06-20 08:07:28+00:00"),
            end_time=datetime.fromisoformat("2025-06-27 13:37:00+00:00")
        )
        bounds2 = fastgpx.TimeBounds(
            start_time=datetime.fromisoformat("2025-06-25 10:00:00+00:00"),
            end_time=datetime.fromisoformat("2025-06-30 15:00:00+00:00")
        )
        bounds1.add(bounds2)
        assert not bounds1.is_empty()
        assert bounds1.is_range()
        assert bounds1.start_time == bounds1.start_time
        assert bounds1.end_time == bounds2.end_time



# Re-export the C extension module to make it available as fastgpx instead of fastgpx.fastgpx.
from .fastgpx import *  # type: ignore

# Needed for sphinx autosummary to pick up the re-exported C extension module.
__all__ = [name for name in dir(fastgpx) if not name.startswith('_')]  # type: ignore



import math
import os
import time
import timeit
import xml.etree.ElementTree as ET
from math import radians, sin, cos, sqrt, atan2
from typing import Callable, TypedDict

import gpxpy
from gpxpy.geo import EARTH_RADIUS
from gpxpy.geo import length_2d, length_3d, Location

from lxml import etree

from colorama import Fore, Back, Style

import fastgpx

# https://github.com/jedie/gpxpy/commit/b371de31826cfecc049a57178d168b04a7f6d0d8
# https://github.com/jedie/gpxpy/blob/b371de31826cfecc049a57178d168b04a7f6d0d8/gpxpy/geo.py#L38
# https://github.com/jedie/gpxpy/blob/b371de31826cfecc049a57178d168b04a7f6d0d8/gpxpy/geo.py#L166
EARTH_RADIUS = 6378.137 * 1000


def to_rad(x: float) -> float:
    return x / 180.0 * math.pi


def haversine(lat1: float, lon1: float, lat2: float, lon2: float) -> float:
    d_lat = to_rad(lat2 - lat1)
    d_lon = to_rad(lon2 - lon1)
    lat1_rad = to_rad(lat1)
    lat2_rad = to_rad(lat2)

    a = math.sin(d_lat / 2) ** 2 + math.cos(lat1_rad) * \
        math.cos(lat2_rad) * math.sin(d_lon / 2) ** 2
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
    return EARTH_RADIUS * c  # Distance in meters


def get_gpx_files(path: str):
    gpx_files = []
    files = os.listdir(path)
    for filename in files:
        if filename.endswith(".gpx"):
            gpx_path = os.path.join(path, filename)
            gpx_files.append(gpx_path)
    return gpx_files


GPX_PATH = '../gpx/2024 Great Roadtrip'

# gpxpy


def read_gpxpy():
    total_length = 0.0
    gpx_files = get_gpx_files(GPX_PATH)
    for gpx_filepath in gpx_files:
        with open(gpx_filepath, 'r', encoding='utf-8') as gpx_file:
            gpx = gpxpy.parse(gpx_file)
            total_length += gpx.length_2d()
            # total_length += gpx.length_3d()
    print('gpxpy', total_length, 'meters')
    return total_length

# xml.etree


def xml_etree_calculate_segment_length(trkseg) -> float:
    points = []
    ns = {'gpx': 'http://www.topografix.com/GPX/1/1'}
    for trkpt in trkseg.findall('.//gpx:trkpt', ns):
        lat = float(trkpt.attrib['lat'])
        lon = float(trkpt.attrib['lon'])
        points.append(Location(lat, lon))
    return length_2d(points)


def xml_etree_calculate_gpx_length(file_path: str) -> float:
    tree = ET.parse(file_path)
    root = tree.getroot()
    ns = {'gpx': 'http://www.topografix.com/GPX/1/1'}
    total_distance = 0.0
    for trk in root.findall('.//gpx:trk', ns):
        for trkseg in trk.findall('.//gpx:trkseg', ns):
            total_distance += xml_etree_calculate_segment_length(trkseg)
    return total_distance


def read_xml_etree():
    total_length = 0.0
    gpx_files = get_gpx_files(GPX_PATH)
    for gpx_filepath in gpx_files:
        length = xml_etree_calculate_gpx_length(gpx_filepath)
        total_length += length
    print('xml_etree', total_length, 'meters')
    return total_length


# lxml


def lxml_calculate_segment_length(trkseg) -> float:
    points = []
    for trkpt in trkseg.findall('.//{http://www.topografix.com/GPX/1/1}trkpt'):
        lat = float(trkpt.attrib['lat'])
        lon = float(trkpt.attrib['lon'])
        points.append(Location(lat, lon))
    return length_2d(points)


def lxml_calculate_gpx_length(file_path: str) -> float:
    tree = etree.parse(file_path, None)
    root = tree.getroot()

    total_distance = 0.0

    for trk in root.findall('.//{http://www.topografix.com/GPX/1/1}trk'):
        for trkseg in trk.findall('.//{http://www.topografix.com/GPX/1/1}trkseg'):
            total_distance += lxml_calculate_segment_length(trkseg)

    return total_distance


def read_lxml():
    total_length = 0.0
    gpx_files = get_gpx_files(GPX_PATH)
    for gpx_filepath in gpx_files:
        length = lxml_calculate_gpx_length(gpx_filepath)
        total_length += length
    print('lxml', total_length, 'meters')
    return total_length


def read_fastgpx():
    total_length = 0.0
    gpx_files = get_gpx_files(GPX_PATH)
    for gpx_filepath in gpx_files:
        fullpath = os.path.abspath(gpx_filepath)
        gpx = fastgpx.parse(fullpath)
        length = gpx.length_2d()
        total_length += length
    print('fastgpx', total_length, 'meters')
    return total_length

# Benchmarks:


class Benchmark(TypedDict):
    name: str
    function: Callable[[], float]


benchmarks: list[Benchmark] = [
    # {'name': 'gpxpy', 'function': read_gpxpy}, # Slow!
    {'name': 'xml_etree', 'function': read_xml_etree},
    {'name': 'lxml', 'function': read_lxml},
    {'name': 'fastgpx', 'function': read_fastgpx},
]

# print(f"Python script PID: {os.getpid()}")
# print("Press Enter to continue...")
# input()

print()
print('Computing expected distance with gpxpy...' + Fore.LIGHTMAGENTA_EX)
# gpxpy is _very_ slow, so omitting from doing multiple benchmark runs with it.
t = time.perf_counter()
expected = read_gpxpy()
d = time.perf_counter()
e = d - t
print(Fore.LIGHTYELLOW_EX + f'gpxpy: {e:.6f} seconds')
print()

iterations = 3
print(Fore.CYAN + f'Running {len(benchmarks)} benchmarks with {iterations} iterations...')
for benchmark in benchmarks:
    func = benchmark['function']
    print()
    # Benchmark timings:
    print(Fore.LIGHTYELLOW_EX + f"Running {benchmark['name']} ..." + Fore.LIGHTBLACK_EX)
    execution_time = timeit.timeit(func, number=iterations)
    average_time = execution_time / iterations
    print(Fore.LIGHTYELLOW_EX +
          f"{benchmark['name']}: {execution_time:.6f} seconds (Average: {average_time:.6f} seconds)" + Fore.YELLOW)
    # Print info about result accuracy:
    result = func()
    name = benchmark['name'].split(' ')[0]
    indent = ' ' * len(name)
    print(Fore.YELLOW +
          f"{indent} {expected} meters (expected)")
    deviation = result - expected
    percent = (deviation / expected) * 100
    print(Fore.LIGHTMAGENTA_EX + f'Distance deviation: {deviation} meters ({percent:.4f}%)')
    if deviation != 0.0:
        print(Back.LIGHTRED_EX + Fore.WHITE + '  FAIL  ' + Back.RESET + Fore.RESET)

print(Fore.RESET)



import gpxpy
import polyline
import pytest

import fastgpx


@pytest.fixture
def gpx_path():
    return "gpx/2024 TopCamp/Connected_20240518_094959_.gpx"


@pytest.fixture
def expected_gpx(gpx_path: str):
    with open(gpx_path, 'r', encoding='utf-8') as gpx_file:
        gpx = gpxpy.parse(gpx_file)
    return gpx

# fastgpx.polyline.encode


def test_encode_p5_segment(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    points = [(point.latitude, point.longitude)
              for point in gpx.tracks[0].segments[0].points]

    expected = polyline.encode(points, precision=5)
    actual = fastgpx.polyline.encode(
        gpx.tracks[0].segments[0].points,
        precision=fastgpx.polyline.Precision.Five)
    assert actual == expected


def test_encode_p6_segment(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    points = [(point.latitude, point.longitude)
              for point in gpx.tracks[0].segments[0].points]

    expected = polyline.encode(points, precision=6)
    actual = fastgpx.polyline.encode(
        gpx.tracks[0].segments[0].points,
        precision=fastgpx.polyline.Precision.Six)
    assert actual == expected


def test_encode_p5_segment_int_overload(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    points = [(point.latitude, point.longitude)
              for point in gpx.tracks[0].segments[0].points]

    expected = polyline.encode(points, precision=5)
    actual = fastgpx.polyline.encode(
        gpx.tracks[0].segments[0].points, precision=5)
    assert actual == expected


def test_encode_p6_segment_int_overload(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    points = [(point.latitude, point.longitude)
              for point in gpx.tracks[0].segments[0].points]

    expected = polyline.encode(points, precision=6)
    actual = fastgpx.polyline.encode(
        gpx.tracks[0].segments[0].points, precision=6)
    assert actual == expected


def test_encode_segment_int_overload_invalid_arguments(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    points = gpx.tracks[0].segments[0].points

    with pytest.raises(ValueError):
        fastgpx.polyline.encode(points, precision=4)

    with pytest.raises(ValueError):
        fastgpx.polyline.encode(points, precision=7)

# fastgpx.polyline.decode


def test_decode_p5_segment(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    points = [(point.latitude, point.longitude)
              for point in gpx.tracks[0].segments[0].points]
    polyline6 = polyline.encode(points, precision=5)

    expected = polyline.decode(polyline6, precision=5)
    result = fastgpx.polyline.decode(polyline6,
                                     precision=fastgpx.polyline.Precision.Five)
    actual = [(point.latitude, point.longitude) for point in result]
    assert actual == expected


def test_decode_p6_segment(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    points = [(point.latitude, point.longitude)
              for point in gpx.tracks[0].segments[0].points]
    polyline6 = polyline.encode(points, precision=6)

    expected = polyline.decode(polyline6, precision=6)
    result = fastgpx.polyline.decode(polyline6,
                                     precision=fastgpx.polyline.Precision.Six)
    actual = [(point.latitude, point.longitude) for point in result]
    assert actual == expected


def test_decode_p5_segment_int_overload(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    points = [(point.latitude, point.longitude)
              for point in gpx.tracks[0].segments[0].points]
    polyline6 = polyline.encode(points, precision=5)

    expected = polyline.decode(polyline6, precision=5)
    result = fastgpx.polyline.decode(polyline6, precision=5)
    actual = [(point.latitude, point.longitude) for point in result]
    assert actual == expected


def test_decode_p6_segment_int_overload(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    points = [(point.latitude, point.longitude)
              for point in gpx.tracks[0].segments[0].points]
    polyline6 = polyline.encode(points, precision=6)

    expected = polyline.decode(polyline6, precision=6)
    result = fastgpx.polyline.decode(polyline6, precision=6)
    actual = [(point.latitude, point.longitude) for point in result]
    assert actual == expected


def test_decode_segment_int_overload_invalid_arguments(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    points = [(point.latitude, point.longitude)
              for point in gpx.tracks[0].segments[0].points]
    polyline6 = polyline.encode(points, precision=6)

    with pytest.raises(ValueError):
        fastgpx.polyline.decode(polyline6, precision=4)

    with pytest.raises(ValueError):
        fastgpx.polyline.decode(polyline6, precision=7)



from __future__ import annotations
from fastgpx.fastgpx import Bounds
from fastgpx.fastgpx import Gpx
from fastgpx.fastgpx import LatLong
from fastgpx.fastgpx import Segment
from fastgpx.fastgpx import TimeBounds
from fastgpx.fastgpx import Track
from fastgpx.fastgpx import geo
from fastgpx.fastgpx import parse
from fastgpx.fastgpx import polyline
from . import fastgpx
__all__: list = ['Bounds', 'Gpx', 'LatLong', 'Segment', 'TimeBounds', 'Track', 'geo', 'parse', 'polyline']



import datetime

import gpxpy
import pytest

import fastgpx


@pytest.fixture
def gpx_path():
    return "gpx/2024 TopCamp/Connected_20240518_094959_.gpx"


@pytest.fixture
def expected_gpx(gpx_path: str):
    with open(gpx_path, 'r', encoding='utf-8') as gpx_file:
        gpx = gpxpy.parse(gpx_file)
    return gpx


METERS_TOL = 1e-4


def test_simple_segment_length2d():
    path = 'gpx/test/debug-segment.gpx'
    gpx = fastgpx.parse(path)
    # Assigning intermediate values for easier debug inspection.
    tracks = gpx.tracks
    track = tracks[0]
    segments = track.segments
    segment = segments[0]
    distance = segment.length_2d()
    assert distance == pytest.approx(1.3839, abs=METERS_TOL)

# fastgpx.Gpx.length_2d


def test_gpx_length2d(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    distance = gpx.length_2d()
    assert distance == pytest.approx(382952.7193, abs=METERS_TOL)


def test_track_length2d(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    distance = gpx.tracks[0].length_2d()
    assert distance == pytest.approx(382952.7193, abs=METERS_TOL)


def test_segment_length2d(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    distance = gpx.tracks[0].segments[0].length_2d()
    assert distance == pytest.approx(17809.2701, abs=METERS_TOL)

# fastgpx.Gpx.name


def test_gpx_name_missing():
    path = 'gpx/test/debug-segment.gpx'
    gpx = fastgpx.parse(path)
    assert gpx.name is None


def test_gpx_name_empty_string(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    assert gpx.name == ''


def test_gpx_name():
    path = 'gpx/test/two-points.gpx'
    gpx = fastgpx.parse(path)
    assert gpx.name == 'Two Point Segment'

# fastgpx.Gpx.bounds


def test_gpx_bounds(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    bounds = gpx.bounds()
    assert not bounds.is_empty()
    assert bounds.min is not None
    assert bounds.min.latitude == pytest.approx(61.410713)
    assert bounds.min.longitude == pytest.approx(10.427408)
    assert bounds.max is not None
    assert bounds.max.latitude == pytest.approx(63.441189)
    assert bounds.max.longitude == pytest.approx(13.142774)

# fastgpx.Gpx.time_bounds


def test_gpx_time_bounds(gpx_path: str):
    gpx = fastgpx.parse(gpx_path)
    time_bounds = gpx.time_bounds()
    assert not time_bounds.is_empty()

    # 2024-05-18T07:50:00Z
    assert time_bounds.start_time is not None
    assert time_bounds.start_time == datetime.datetime(
        year=2024, month=5, day=18, hour=7, minute=50, second=0, tzinfo=datetime.timezone.utc)
    assert time_bounds.start_time.tzinfo is datetime.timezone.utc
    assert time_bounds.start_time.isoformat() == '2024-05-18T07:50:00+00:00'
    assert time_bounds.start_time.timestamp() == 1716018600

    # 2024-05-18T16:46:18Z
    assert time_bounds.end_time is not None
    assert time_bounds.end_time == datetime.datetime(
        year=2024, month=5, day=18, hour=16, minute=46, second=18, tzinfo=datetime.timezone.utc)
    assert time_bounds.end_time.tzinfo is datetime.timezone.utc
    assert time_bounds.end_time.isoformat() == '2024-05-18T16:46:18+00:00'
    assert time_bounds.end_time.timestamp() == 1716050778



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

import re
import os
import sys
import tomllib
import logging as std_logging
from sphinx.util import logging

PROJECT_ROOT = os.path.abspath('../../')

sys.path.insert(0, os.path.join(PROJECT_ROOT, 'src'))

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

# Read the version from pyproject.toml
with open(os.path.join(PROJECT_ROOT, 'pyproject.toml'), 'rb') as f:
    pyproject = tomllib.load(f)
    pyproject_name = pyproject['project']['name']
    pyproject_version = pyproject['project']['version']

assert pyproject['project']['name'] is not None
assert pyproject['project']['version'] is not None

project = pyproject['project']['name']
author = 'Thomas Thomassen'
copyright = f'2024-2025, {author}'
release = pyproject['project']['version']

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.napoleon',  # For NumPy and Google style docstrings
    'sphinx_rtd_theme',
]

templates_path = ['_templates']
# exclude_patterns = []

autosummary_generate = True


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_rtd_theme'
html_static_path = ['_static']


# -- Adjust formatting for overloaded methods --------------------------------

logger = logging.getLogger(__name__)


def format_overloaded_docstring(app, what, name, obj, options, lines):
    logger.info(f"[fastgpx-doc] Processing docstring for {name}")
    if not lines:
        return

    logger.info(f"[fastgpx-doc] First line: {lines[0]}")
    if lines[0].strip().startswith("Overloaded function"):
        logger.info(f"[fastgpx-doc] Detected overloaded function in {name}")

        new_lines = [".. rubric:: Overloads", ""]

        for line in lines[1:]:
            match = re.match(r"^\d+\.\s+(.*)", line.strip())
            if match:
                new_lines.extend([
                    ".. code-block:: python",
                    "",
                    f"    {match.group(1)}",
                    ""
                ])

        lines.clear()
        lines.extend(new_lines)


def setup(app):
    if app.verbosity >= 1:
        logger.setLevel(std_logging.INFO)
    else:
        logger.setLevel(std_logging.WARNING)

    logger.info("[fastgpx-doc] Setting up overloaded docstring formatting")
    app.connect("autodoc-process-docstring", format_overloaded_docstring)



from __future__ import annotations
import datetime
import typing
from . import geo
from . import polyline
__all__ = ['Bounds', 'Gpx', 'LatLong', 'Segment', 'TimeBounds', 'Track', 'geo', 'parse', 'polyline']
class Bounds:
    max: LatLong | None
    min: LatLong | None
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, min: LatLong, max: LatLong) -> None:
        ...
    @typing.overload
    def __init__(self, min: tuple[float, float], max: tuple[float, float]) -> None:
        ...
    @typing.overload
    def add(self, location: LatLong) -> None:
        ...
    @typing.overload
    def add(self, bounds: Bounds) -> None:
        ...
    def is_empty(self) -> bool:
        ...
    def max_bounds(self, bounds: Bounds) -> Bounds:
        ...
    @property
    def max_latitude(self) -> float | None:
        ...
    @max_latitude.setter
    def max_latitude(self, arg1: float) -> None:
        ...
    @property
    def max_longitude(self) -> float | None:
        ...
    @max_longitude.setter
    def max_longitude(self, arg1: float) -> None:
        ...
    @property
    def min_latitude(self) -> float | None:
        ...
    @min_latitude.setter
    def min_latitude(self, arg1: float) -> None:
        ...
    @property
    def min_longitude(self) -> float | None:
        ...
    @min_longitude.setter
    def min_longitude(self, arg1: float) -> None:
        ...
class Gpx:
    name: str | None
    tracks: list[Track]
    def __init__(self) -> None:
        ...
    def bounds(self) -> Bounds:
        ...
    def get_bounds(self) -> Bounds:
        ...
    def get_time_bounds(self) -> TimeBounds:
        ...
    def length_2d(self) -> float:
        ...
    def length_3d(self) -> float:
        ...
    def time_bounds(self) -> TimeBounds:
        ...
class LatLong:
    elevation: float
    latitude: float
    longitude: float
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, latitude: float, longitude: float, elevation: float = 0.0) -> None:
        ...
class Segment:
    points: list[LatLong]
    def __init__(self) -> None:
        ...
    def bounds(self) -> Bounds:
        ...
    def get_bounds(self) -> Bounds:
        ...
    def get_time_bounds(self) -> TimeBounds:
        ...
    def length_2d(self) -> float:
        ...
    def length_3d(self) -> float:
        ...
    def time_bounds(self) -> TimeBounds:
        ...
class TimeBounds:
    end_time: datetime.datetime | None
    start_time: datetime.datetime | None
    @typing.overload
    def __init__(self) -> None:
        ...
    @typing.overload
    def __init__(self, start_time: datetime.datetime | None, end_time: datetime.datetime | None) -> None:
        ...
    @typing.overload
    def add(self, datetime: datetime.datetime) -> None:
        ...
    @typing.overload
    def add(self, timebounds: TimeBounds) -> None:
        ...
    def is_empty(self) -> bool:
        ...
    def is_range(self) -> bool:
        ...
class Track:
    comment: str | None
    description: str | None
    name: str | None
    number: int | None
    segments: list[Segment]
    type: str | None
    def __init__(self) -> None:
        ...
    def bounds(self) -> Bounds:
        ...
    def get_bounds(self) -> Bounds:
        ...
    def get_time_bounds(self) -> TimeBounds:
        ...
    def length_2d(self) -> float:
        ...
    def length_3d(self) -> float:
        ...
    def time_bounds(self) -> TimeBounds:
        ...
def parse(path: str) -> Gpx:
    ...



from __future__ import annotations
import fastgpx.fastgpx
__all__ = ['haversine', 'haversine_distance']
def haversine(latlong1: fastgpx.fastgpx.LatLong, latlong2: fastgpx.fastgpx.LatLong) -> float:
    ...
def haversine_distance(latitude_1: float, longitude_1: float, latitude_2: float, longitude_2: float) -> float:
    """
    Compatibility with `gpxpy.geo.haversine_distance`
    """



import fastgpx

# fastgpx.Bounds.__init__


def test_bounds_init_defaults():
    bounds = fastgpx.Bounds()
    assert bounds.is_empty()
    assert bounds.min is None
    assert bounds.min is None
    assert bounds.max is None
    assert bounds.max is None


def test_bounds_init_with_latlongs():
    ll1 = fastgpx.LatLong(-10, -5)
    ll2 = fastgpx.LatLong(30, 25)
    bounds = fastgpx.Bounds(ll1, ll2)
    assert not bounds.is_empty()
    assert bounds.min is not None
    assert bounds.max is not None

    assert bounds.min.latitude == -10.0
    assert bounds.min.longitude == -5.0
    assert bounds.max.latitude == 30.0
    assert bounds.max.longitude == 25.0

    assert bounds.min.latitude == bounds.min_latitude
    assert bounds.min.longitude == bounds.min_longitude
    assert bounds.max.latitude == bounds.max_latitude
    assert bounds.max.longitude == bounds.max_longitude


def test_bounds_init_with_tuples():
    bounds = fastgpx.Bounds((-10, -5), (30, 25))
    assert not bounds.is_empty()
    assert bounds.min is not None
    assert bounds.min.latitude == -10.0
    assert bounds.min.longitude == -5.0
    assert bounds.max is not None
    assert bounds.max.latitude == 30.0
    assert bounds.max.longitude == 25.0

# fastgpx.Bounds.test_bounds_max_bounds


def test_bounds_max_bounds():
    bounds1 = fastgpx.Bounds((-10, -5), (30, 25))
    bounds2 = fastgpx.Bounds((5, 30), (40, 20))
    bounds = bounds1.max_bounds(bounds2)
    assert not bounds.is_empty()
    assert bounds.min is not None
    assert bounds.min.latitude == -10.0
    assert bounds.min.longitude == -5.0
    assert bounds.max is not None
    assert bounds.max.latitude == 40.0
    assert bounds.max.longitude == 30.0
